// mark as Protocol Buffers v2 format
syntax = "proto2";

// Examples from https://protobuf.dev/docs/encoding
package protobuf_encoding_test;

message Test1 {
  required int32 a = 1;
}
message Test2 {
  required string b = 2;
}
message Test3 {
  required Test1 c = 3;
}
message Test4 {
  repeated int32 d = 4 [packed=true];
}
