#ifndef FIELD_TYPES_H
#define FIELD_TYPES_H

using namespace google::protobuf;

#define TYPE_DOUBLE FieldDescriptor::TYPE_DOUBLE
#define TYPE_FLOAT FieldDescriptor::TYPE_FLOAT
#define TYPE_INT64 FieldDescriptor::TYPE_INT64
#define TYPE_UINT64 FieldDescriptor::TYPE_UINT64
#define TYPE_INT32 FieldDescriptor::TYPE_INT32
#define TYPE_FIXED64 FieldDescriptor::TYPE_FIXED64
#define TYPE_FIXED32 FieldDescriptor::TYPE_FIXED32
#define TYPE_BOOL FieldDescriptor::TYPE_BOOL
#define TYPE_STRING FieldDescriptor::TYPE_STRING
#define TYPE_GROUP FieldDescriptor::TYPE_GROUP
#define TYPE_MESSAGE FieldDescriptor::TYPE_MESSAGE
#define TYPE_BYTES FieldDescriptor::TYPE_BYTES
#define TYPE_UINT32 FieldDescriptor::TYPE_UINT32
#define TYPE_ENUM FieldDescriptor::TYPE_ENUM
#define TYPE_SFIXED32 FieldDescriptor::TYPE_SFIXED32
#define TYPE_SFIXED64 FieldDescriptor::TYPE_SFIXED64
#define TYPE_SINT32 FieldDescriptor::TYPE_SINT32
#define TYPE_SINT64 FieldDescriptor::TYPE_SINT64
#define MAX_TYPE FieldDescriptor::MAX_TYPE

#define CPPTYPE_INT32 FieldDescriptor::CPPTYPE_INT32
#define CPPTYPE_INT64 FieldDescriptor::CPPTYPE_INT64
#define CPPTYPE_UINT32 FieldDescriptor::CPPTYPE_UINT32
#define CPPTYPE_UINT64 FieldDescriptor::CPPTYPE_UINT64
#define CPPTYPE_DOUBLE FieldDescriptor::CPPTYPE_DOUBLE
#define CPPTYPE_FLOAT FieldDescriptor::CPPTYPE_FLOAT
#define CPPTYPE_BOOL FieldDescriptor::CPPTYPE_BOOL
#define CPPTYPE_ENUM FieldDescriptor::CPPTYPE_ENUM
#define CPPTYPE_STRING FieldDescriptor::CPPTYPE_STRING
#define CPPTYPE_MESSAGE FieldDescriptor::CPPTYPE_MESSAGE
#define MAX_CPPTYPE FieldDescriptor::MAX_CPPTYPE

#endif
